@@ -21,6 +21,9 @@ extern "C" {
 #include <cassert>
 #include <set>
 #include <vector>
+#include <algorithm>
+#include <iterator>
+
 
 using namespace std;
 
@@ -39,9 +42,13 @@ lex_cast(IN const & in)
 // ------------------------------------------------------------------------
 
 
+unsigned derived_probe::last_probeidx = 0;
+
 derived_probe::derived_probe (probe *p):
   base (p)
 {
+  this->name = string ("probe_") + lex_cast<string>(last_probeidx ++);
+
   if (p)
     {
       this->locations = p->locations;  
@@ -54,8 +61,11 @@ derived_probe::derived_probe (probe *p):
 derived_probe::derived_probe (probe *p, probe_point *l):
   base (p)
 {
+  this->name = string ("probe_") + lex_cast<string>(last_probeidx ++);
+
   if (l)
     this->locations.push_back (l);
+
   if (p)
     {
       this->tok = p->tok;
@@ -1240,7 +1250,7 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
           {
             if (s.verbose>2)
               clog << "Eliding unused local variable "
-                   << l->name << " in probe #" << i << endl;
+                   << l->name << " in " << s.probes[i]->name << endl;
             s.probes[i]->locals.erase(s.probes[i]->locals.begin() + j);
             relaxed_p = false;
             // don't increment j
@@ -1299,7 +1309,10 @@ struct dead_assignment_remover: public traversing_visitor
     session(s), relaxed_p(r), vut(v), current_expr(0) {}
 
   void visit_expr_statement (expr_statement* s);
-  // XXX: and other places where an assignment may be nested
+  // XXX: other places where an assignment may be nested should be
+  // handled too (e.g., loop/if conditionals, array indexes, function
+  // parameters).  Until then, they result in visit_assignment() being
+  // called with null current_expr.
 
   void visit_assignment (assignment* e);
 };
@@ -1321,7 +1334,9 @@ dead_assignment_remover::visit_assignment (assignment* e)
 {
   symbol* left = get_symbol_within_expression (e->left);
   vardecl* leftvar = left->referent;
-  if (*current_expr == e) // we're not nested any deeper than expected 
+  if (current_expr && // see XXX above: this case represents a missed
+                      // optimization opportunity
+      *current_expr == e) // we're not nested any deeper than expected 
     {
       // clog << "Checking assignment to " << leftvar->name << " at " << *e->tok << endl;
       if (vut.read.find(leftvar) == vut.read.end()) // var never read?
@@ -1367,6 +1382,7 @@ struct dead_stmtexpr_remover: public traversing_visitor
   systemtap_session& session;
   bool& relaxed_p;
   statement** current_stmt; // pointer to current stmt* being iterated
+  set<vardecl*> focal_vars; // vars considered subject to side-effects
 
   dead_stmtexpr_remover(systemtap_session& s, bool& r): 
     session(s), relaxed_p(r), current_stmt(0) {}
@@ -1409,7 +1425,23 @@ dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
 
   varuse_collecting_visitor vut;
   s->value->visit (& vut);
-  if (vut.written.empty() && !vut.embedded_seen)
+
+  // Note that side-effect-freeness is not simply this test:
+  //
+  // (vut.written.empty() && !vut.embedded_seen)
+  //
+  // That's because the vut.written list may consist of local
+  // variables of called functions.  Visible side-effects occur if
+  // *our* locals, or any *globals* are written-to.
+
+
+  set<vardecl*> intersection;
+  insert_iterator<set<vardecl*> > int_it (intersection, intersection.begin());
+  set_intersection (vut.written.begin(), vut.written.end(),
+                    focal_vars.begin(), focal_vars.end(),
+                    int_it);
+
+  if (intersection.empty() && ! vut.embedded_seen)
     {
       if (session.verbose>2)
         clog << "Eliding side-effect-free expression "
@@ -1425,6 +1457,15 @@ dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
       
       relaxed_p = false;
     }
+  else if(session.verbose>3)
+    {
+      clog << "keeping expression " << *s->tok 
+           << " because it writes: ";
+      for (std::set<vardecl*>::iterator k = intersection.begin();
+           k != intersection.end(); k++)
+        clog << (*k)->name << " ";
+      clog << "and/or embedded: " << vut.embedded_seen << endl;
+    }  
 }
 
 
@@ -1438,9 +1479,25 @@ void semantic_pass_opt4 (systemtap_session& s, bool& relaxed_p)
   // This instance may be reused for multiple probe/function body trims.
 
   for (unsigned i=0; i<s.probes.size(); i++)
-    s.probes[i]->body->visit (& duv);
+    {
+      duv.focal_vars.clear ();
+      duv.focal_vars.insert (s.globals.begin(),
+                             s.globals.end());
+      duv.focal_vars.insert (s.probes[i]->locals.begin(),
+                             s.probes[i]->locals.end());
+      s.probes[i]->body->visit (& duv);
+    }
   for (unsigned i=0; i<s.functions.size(); i++)
-    s.functions[i]->body->visit (& duv);
+    {
+      duv.focal_vars.clear ();
+      duv.focal_vars.insert (s.functions[i]->locals.begin(),
+                             s.functions[i]->locals.end());
+      duv.focal_vars.insert (s.functions[i]->formal_args.begin(),
+                             s.functions[i]->formal_args.end());
+      duv.focal_vars.insert (s.globals.begin(),
+                             s.globals.end());
+      s.functions[i]->body->visit (& duv);
+    }
 }
 
 
@@ -1502,6 +1559,7 @@ semantic_pass_types (systemtap_session& s)
       for (unsigned j=0; j<s.functions.size(); j++)
         {
           functiondecl* fn = s.functions[j];
+          ti.current_probe = 0;
           ti.current_function = fn;
           ti.t = pe_unknown;
           fn->body->visit (& ti);
@@ -1517,6 +1575,7 @@ semantic_pass_types (systemtap_session& s)
         {
           derived_probe* pn = s.probes[j];
           ti.current_function = 0;
+          ti.current_probe = pn;
           ti.t = pe_unknown;
           pn->body->visit (& ti);
         }
@@ -1544,6 +1603,13 @@ semantic_pass_types (systemtap_session& s)
 }
 
 
+
+typeresolution_info::typeresolution_info (systemtap_session& s):
+  session(s), current_function(0), current_probe(0)
+{
+}
+
+
 void
 typeresolution_info::visit_literal_number (literal_number* e)
 {
@@ -1962,9 +2028,6 @@ typeresolution_info::visit_functioncall (functioncall* e)
   if (e->type == pe_stats)
     invalid (e->tok, e->type);
 
-  // XXX: but what about functions that return no value,
-  // and are used only as an expression-statement for side effects?
-
   // now resolve the function parameters
   if (e->args.size() != e->referent->formal_args.size())
     unresolved (e->tok); // symbol resolution should prevent this
@@ -2247,7 +2310,8 @@ typeresolution_info::visit_print_format (print_format* e)
 	  if (e->components[i].type == print_format::conv_unspecified)
 	    throw semantic_error ("Unspecified conversion in print operator format string",
 				  e->tok);
-	  else if (e->components[i].type == print_format::conv_literal)
+	  else if (e->components[i].type == print_format::conv_literal 
+		   || e->components[i].type == print_format::conv_size)
 	    continue;
 	  components.push_back(e->components[i]);
 	}
@@ -2267,9 +2331,9 @@ typeresolution_info::visit_print_format (print_format* e)
 
 	  switch (components[i].type)
 	    {
-
 	    case print_format::conv_unspecified:
 	    case print_format::conv_literal:
+	    case print_format::conv_size:
 	      assert (false);
 	      break;
 
@@ -2279,6 +2343,7 @@ typeresolution_info::visit_print_format (print_format* e)
 	    case print_format::conv_unsigned_ptr:
 	    case print_format::conv_unsigned_uppercase_hex:
 	    case print_format::conv_unsigned_lowercase_hex:
+	    case print_format::conv_binary:
 	      wanted = pe_long;
 	      break;
 
@@ -2367,7 +2432,10 @@ typeresolution_info::unresolved (const token* tok)
 
   if (assert_resolvability)
     {
-      cerr << "semantic error: unresolved type for ";
+      string nm = (current_function ? current_function->name :
+                   current_probe ? current_probe->name :
+                   "?");
+      cerr << "semantic error: " + nm + " with unresolved type for ";
       if (tok)
         cerr << *tok;
       else
@@ -2384,7 +2452,10 @@ typeresolution_info::invalid (const token* tok, exp_type pe)
 
   if (assert_resolvability)
     {
-      cerr << "semantic error: invalid type " << pe << " for ";
+      string nm = (current_function ? current_function->name :
+                   current_probe ? current_probe->name :
+                   "?");
+      cerr << "semantic error: " + nm + " with invalid type " << pe << " for ";
       if (tok)
         cerr << *tok;
       else
@@ -2401,7 +2472,10 @@ typeresolution_info::mismatch (const token* tok, exp_type t1, exp_type t2)
 
   if (assert_resolvability)
     {
-      cerr << "semantic error: type mismatch for ";
+      string nm = (current_function ? current_function->name :
+                   current_probe ? current_probe->name :
+                   "?");
+      cerr << "semantic error: " + nm + " with type mismatch for ";
       if (tok)
         cerr << *tok;
       else
