@@ -47,12 +47,13 @@ public:
 
 struct typeresolution_info: public visitor
 {
-  typeresolution_info (systemtap_session& s): session (s) {}
+  typeresolution_info (systemtap_session& s);
   systemtap_session& session;
   unsigned num_newly_resolved;
   unsigned num_still_unresolved;
   bool assert_resolvability;
   functiondecl* current_function;
+  derived_probe* current_probe;
 
   void mismatch (const token* tok, exp_type t1, exp_type t2);
   void unresolved (const token* tok);
@@ -112,20 +113,23 @@ struct derived_probe: public probe
   derived_probe (probe* b, probe_point* l);
   probe* base; // the original parsed probe
 
+private:
+  static unsigned last_probeidx;
+
+public:
+  std::string name;
+
   virtual ~derived_probe () {}
 
-  virtual void emit_registrations (translator_output* o,
-                                   unsigned probeidx) = 0;
+  virtual void emit_registrations (translator_output* o) = 0;
   // (from within module_init):
-  // rc = ..... ENTRYFN ;
+  // rc = ..... register_or_whatever (ENTRYFN);
 
-  virtual void emit_deregistrations (translator_output* o,
-                                     unsigned probeidx) = 0;
+  virtual void emit_deregistrations (translator_output* o) = 0;
   // (from within module_exit):
-  // rc = ..... ENTRYFN ;
+  // (void) ..... unregister_or_whatever (ENTRYFN);
 
-  virtual void emit_probe_entries (translator_output* o,
-                                   unsigned probeidx) = 0;
+  virtual void emit_probe_entries (translator_output* o) = 0;
   // ... for all probe-points:
   // ELABORATE_SPECIFIC_SIGNATURE ENTRYFN {
   //   /* allocate context - probe_prologue */
@@ -134,6 +138,11 @@ struct derived_probe: public probe
   //   /* deallocate context - probe_epilogue */
   // }
 
+  virtual void emit_probe_context_vars (translator_output* o) {}
+  // From within unparser::emit_common_header, add any extra variables
+  // to this probe's context locals.
+
+protected:
   void emit_probe_prologue (translator_output* o, const std::string&);
   void emit_probe_epilogue (translator_output* o);
 };
