@@ -91,6 +91,7 @@ usage (systemtap_session& s)
     << "   -c CMD     start the probes, run CMD, and exit when it finishes"
     << endl
     << "   -x PID     sets target() to PID" << endl
+    << "   -t         benchmarking timing information generated" << endl
     ;
   // -d: dump safety-related external references
 
@@ -124,6 +125,7 @@ main (int argc, char * const argv [])
   s.kernel_release = string (buf.release);
   s.architecture = string (buf.machine);
   s.verbose = 0;
+  s.timing = 0;
   s.guru_mode = false;
   s.bulk_mode = false;
   s.unoptimized = false;
@@ -149,7 +151,7 @@ main (int argc, char * const argv [])
 
   while (true)
     {
-      int grc = getopt (argc, argv, "hVvp:I:e:o:R:r:m:kgc:x:D:bs:u");
+      int grc = getopt (argc, argv, "hVvtp:I:e:o:R:r:m:kgc:x:D:bs:u");
       if (grc < 0)
         break;
       switch (grc)
@@ -162,6 +164,10 @@ main (int argc, char * const argv [])
 	  s.verbose ++;
 	  break;
 
+        case 't':
+	  s.timing ++;
+	  break;
+
         case 'p':
           s.last_pass = atoi (optarg);
           if (s.last_pass < 1 || s.last_pass > 5)
@@ -275,14 +281,18 @@ main (int argc, char * const argv [])
   // Create a temporary directory to build within.
   // Be careful with this, as "s.tmpdir" is "rm -rf"'d at the end.
   {
-    char tmpdirt[] = "/tmp/stapXXXXXX";
-    const char* tmpdir = mkdtemp (tmpdirt);
+    char* tmpdir_env = getenv("TMPDIR");
+    if (! tmpdir_env)
+      tmpdir_env = "/tmp";
+    
+    string stapdir = "/stapXXXXXX";
+    string tmpdirt = tmpdir_env + stapdir;
+    const char* tmpdir = mkdtemp((char *)tmpdirt.c_str());
     if (! tmpdir)
       {
         const char* e = strerror (errno);
-        cerr << "mkdtemp (\"" << tmpdir << "\"): " << e << endl;
-        s.tmpdir = "";
-        rc = 1;
+        cerr << "ERROR: cannot create temporary directory (\"" << tmpdirt << "\"): " << e << endl;
+        exit (1); // die
       }
     else
       s.tmpdir = tmpdir;
