@@ -59,6 +59,8 @@ static char proc_filebase[128];
 static int proc_file[NR_CPUS];
 
 /* probe output written here, if non-NULL */
+/* if no output file name is specified, use this */
+#define DEFAULT_RELAYFS_OUTFILE_NAME "probe.out"
 extern char *outfile_name;
 
 /* internal variables */
@@ -577,6 +579,9 @@ static int merge_output(void)
 			num[i] = 0;
 	}
 
+	if (!outfile_name)
+		outfile_name = DEFAULT_RELAYFS_OUTFILE_NAME;
+
 	ofp = fopen (outfile_name, "w");
 	if (!ofp) {
 		fprintf (stderr, "ERROR: couldn't open output file %s: errcode = %s\n",
@@ -621,7 +626,7 @@ static int merge_output(void)
 		fclose (fp[i]);
 	fclose (ofp);
 	if (dropped)
-		printf ("\033[33mSequence had %d drops.\033[0m\n", dropped);
+		fprintf (stderr, "Sequence had %d drops.\n", dropped);
 	return 0;
 }
 
@@ -748,19 +753,18 @@ int stp_main_loop(void)
 		switch (type) {
 		case STP_TRANSPORT_INFO:
 		{
-			struct transport_info *info = (struct transport_info *)data;
-
+			struct transport_info *info = (struct transport_info *)data;			
 			transport_mode = info->transport_mode;
 			params.subbuf_size = info->subbuf_size;
 			params.n_subbufs = info->n_subbufs;
 #ifdef DEBUG
 			if (transport_mode == STP_TRANSPORT_RELAYFS)
-				printf ("TRANSPORT_INFO recvd: RELAYFS %d bufs of %d bytes.\n", 
-					params.n_subbufs, 
-					params.subbuf_size);
+				fprintf (stderr, "TRANSPORT_INFO recvd: RELAYFS %d bufs of %d bytes.\n", 
+					 params.n_subbufs, 
+					 params.subbuf_size);
 			else
-				printf ("TRANSPORT_INFO recvd: PROC with %d Mbyte buffers.\n", 
-					info->buf_size); 
+				fprintf (stderr, "TRANSPORT_INFO recvd: PROC with %d Mbyte buffers.\n", 
+					 info->buf_size); 
 #endif
 			if (!streaming()) {
 				rc = init_relayfs();
@@ -784,7 +788,7 @@ int stp_main_loop(void)
 			break;
 		}
 		case STP_REALTIME_DATA:
-			fputs ((char *)data, ofp);
+			fwrite_unlocked(data, nb - sizeof(int), 1, ofp);
 			break;
 		case STP_OOB_DATA:
 			fputs ((char *)data, stderr);
