@@ -281,7 +281,9 @@ struct print_format: public expression
       conv_unsigned_uppercase_hex,
       conv_unsigned_lowercase_hex,
       conv_string,
-      conv_literal
+      conv_literal,
+      conv_binary,
+      conv_size
     };
 
   struct format_component
